# encoding: utf-8

import operator
from functools import reduce
from urllib.parse import parse_qsl

from django.apps import apps
from django.contrib.admin.utils import (
    lookup_needs_distinct, prepare_lookup_value,
)
from django.db.models import Q
from django.http import Http404, JsonResponse
from django.views.generic.list import BaseListView

from cool.core.utils import construct_search
from cool.settings import cool_settings


class CoolAutocompleteJsonView(BaseListView):
    paginate_by = 20

    def get(self, request, *args, **kwargs):
        """
        Return a JsonResponse with search results of the form:
        {
            results: [{id: "123" text: "foo"}],
            pagination: {more: true}
        }
        """
        app_label = request.GET['app_label']
        model_name = request.GET['model_name']
        limit_choices_to = request.GET.get('limit_choices_to', None)
        to_field_name = request.GET.get('to_field_name', 'pk')
        term = request.GET.get('term', '')

        if not self.has_perm(request, app_label, model_name):
            return JsonResponse({'error': '403 Forbidden'}, status=403)
        try:
            self.model = apps.get_model(app_label, model_name)
        except LookupError:
            return JsonResponse({'error': '403 Forbidden'}, status=403)

        queryset = self.get_queryset()

        filters = dict()
        if limit_choices_to:
            for key, value in parse_qsl(limit_choices_to):
                filters[key] = prepare_lookup_value(key, value)
        if filters:
            queryset = queryset.filter(**filters)

        queryset, search_use_distinct = self.get_search_results(self.request, queryset, term)
        if search_use_distinct:
            queryset = queryset.distinct()
        context = self.get_context_data(object_list=queryset)
        return JsonResponse({
            'results': [
                {'id': str(getattr(obj, to_field_name)), 'text': str(obj)}
                for obj in context['object_list']
            ],
            'pagination': {'more': context['page_obj'].has_next()},
        })

    def get_search_results(self, request, queryset, search_term):
        """
        Return a tuple containing a queryset to implement the search
        and a boolean indicating if the results may contain duplicates.
        """
        use_distinct = False
        get_search_fields = getattr(self.model, 'get_search_fields')
        search_fields = None
        if get_search_fields and callable(get_search_fields):
            search_fields = get_search_fields()
        if not search_fields:
            raise Http404('%s must have get_search_fields for the autocomplete_view.' % type(self.model).__name__)

        if search_fields and search_term:
            orm_lookups = [construct_search(queryset, str(search_field)) for search_field in search_fields]
            for bit in search_term.split():
                or_queries = [Q(**{orm_lookup: bit}) for orm_lookup in orm_lookups]
                queryset = queryset.filter(reduce(operator.or_, or_queries))
            use_distinct |= any(lookup_needs_distinct(self.model._meta, search_spec) for search_spec in orm_lookups)

        return queryset, use_distinct

    @classmethod
    def has_perm(cls, request, app_label, model_name):
        if cool_settings.AUTOCOMPLETE_CHECK_PERM:
            return (
                    request.user.has_perm('%s.%s_%s' % (app_label, 'view', model_name)) or
                    request.user.has_perm('%s.%s_%s' % (app_label, 'change', model_name))
            )
        else:
            return request.user.is_staff and request.user.is_active



# encoding: utf-8

from cool.admin.admin import (
    BaseModelAdmin, StrictInlineFormSet, StrictModelFormSet, site_register,
)
from cool.admin.filters import (
    AutocompleteFieldFilter, ContainsFilter, DateRangeFieldFilter,
    IContainsFilter, ISearchFilter, IStartswithFilter, NumberRangeFieldFilter,
    SearchFilter, StartswithFilter,
)

__all__ = [
    'BaseModelAdmin', 'StrictInlineFormSet', 'StrictModelFormSet', 'site_register', 'AutocompleteFieldFilter',
    'ContainsFilter', 'DateRangeFieldFilter', 'IContainsFilter', 'ISearchFilter', 'IStartswithFilter',
    'NumberRangeFieldFilter', 'SearchFilter', 'StartswithFilter',
]



# encoding: utf-8

import enum


class ConstantsMeta(enum.EnumMeta):

    def __new__(metacls, classname, bases, classdict):
        cls = super().__new__(metacls, classname, bases, classdict)
        return enum.unique(cls)


class ConstantsItemWrapper:
    def __init__(self, value):
        self.value = value
        assert isinstance(self.value, tuple) and len(self.value) > 0

    def __eq__(self, other):
        if isinstance(other, ConstantsItemWrapper):
            other = other.value
        if not isinstance(other, tuple) or not len(self.value) > 0:
            return False
        return self.value[0] == other[0]

    def __repr__(self):
        return repr(self.value)


class ConstantsItem:
    def __init__(self, code, desc):
        self.code = code
        self.desc = desc
        if hasattr(self, '_value_'):
            self._value_ = ConstantsItemWrapper(self._value_)

    def __str__(self):
        return str(self.code)

    def __eq__(self, other):
        if type(other) is self.__class__:
            return self.code == other.code
        elif isinstance(other, type(self.code)):
            return self.code == other
        else:
            return False

    def get_tuple(self):
        return self.code, self.desc

    def get_dict(self, **kwargs):
        ret = kwargs.copy()
        ret.update(code=self.code, desc=self.desc)
        return ret


class Constants(ConstantsItem, enum.Enum, metaclass=ConstantsMeta):
    """

        class TestConstants(Constants):
            TEST0 = (0, _('test0desc'))
            TEST1 = (1, _('test desc'))
            TEST2 = (2, _('test desc'))
        InlineConstants = Enum('InlineConstants', (('a', (1,2)), ('b', (3,4))))
    """

    @classmethod
    def _missing_(cls, value):
        return ConstantsItem(*value)

    @classmethod
    def get_choices_list(cls):
        """
        get [(code1, desc1), (code2, desc2)] for choices params
        """
        return [item.get_tuple() for item in cls]

    @classmethod
    def get_desc_dict(cls, name_key='tag'):
        return [item.get_dict(**{name_key: item.name}) for item in cls]



# encoding: utf-8

from cool.views.error_code import ErrorCode
from cool.views.exceptions import CoolAPIException, CoolPermissionAPIException
from cool.views.response import ResponseData
from cool.views.serializer import BaseSerializer
from cool.views.sites import ViewSite
from cool.views.utils import get_api_doc, get_api_info
from cool.views.view import CoolBFFAPIView, PageMixin

__all__ = [
    'ErrorCode', 'CoolAPIException', 'CoolPermissionAPIException', 'ResponseData', 'BaseSerializer', 'ViewSite',
    'get_api_doc', 'get_api_info', 'CoolBFFAPIView', 'PageMixin',
]



# encoding: utf-8
import copy
import inspect
from collections import OrderedDict

from rest_framework.exceptions import ValidationError
from rest_framework.fields import CharField


class SplitCharField(CharField):
    child = CharField(allow_blank=True)

    def __init__(self, **kwargs):
        self.sep = kwargs.pop('sep', ',')
        self.child = kwargs.pop('child', copy.deepcopy(self.child))

        assert not inspect.isclass(self.child), '`child` has not been instantiated.'
        assert self.child.source is None, (
            "The `source` argument is not meaningful when applied to a `child=` field. "
            "Remove `source=` from the field declaration."
        )

        super().__init__(**kwargs)
        self.child.bind(field_name='', parent=self)

    def to_internal_value(self, data):
        if data is None:
            return data
        ret = super().to_internal_value(data)
        return self.run_child_validation(ret.split(self.sep))

    def run_child_validation(self, data):
        result = []
        errors = OrderedDict()

        for idx, item in enumerate(data):
            try:
                result.append(self.child.run_validation(item))
            except ValidationError as e:
                errors[idx] = e.detail

        if not errors:
            return result
        raise ValidationError([errors])

    def to_representation(self, value):
        return self.sep.join(map(str, value))



# encoding: utf-8

from django.db import models

from cool.model import descriptors


class ForeignKey(models.ForeignKey):
    forward_related_accessor_class = descriptors.ForwardManyToOneCacheDescriptor


class OneToOneField(models.OneToOneField):
    forward_related_accessor_class = descriptors.ForwardOneToOneCacheDescriptor



# encoding: utf-8

from django.utils.translation import gettext as _

from cool.core.constants import Constants
from cool.settings import cool_settings

DEFAULT_ERROR_CODES = (
    ('SUCCESS', (0, _('Success'))),

    ('ERROR_UNKNOWN', (-1, _('Unknown Error'))),
    ('ERROR_SYSTEM', (-2, _('System Error'))),

    ('ERROR_BAD_PARAMETER', (-11, _('Bad Parameter Error'))),
    ('ERROR_BAD_FORMAT', (-12, _('Bad Format Error'))),
    ('ERROR_PERMISSION', (-13, _('Permission Error'))),
)

ErrorCode = Constants('ErrorCode', DEFAULT_ERROR_CODES + tuple(cool_settings.API_ERROR_CODES))



# encoding: utf-8


from collections import OrderedDict
from urllib.parse import urljoin

from django import forms, http
from django.views.generic import View as DjangoView

from cool.core.utils import split_camel_name
from cool.views.param import Param


def _camel2path(name):
    """GenerateURLs => generate/urls"""
    return '/'.join(split_camel_name(name)).lower() + '/'


def _size_len(text):
    return (len(text) + len(text.encode('utf8'))) // 2


def _format(text, size):
    return text + (" " * (size - _size_len(text)))


class ViewOptions:
    """View options class

    * name              resolver name of the URL patterns, default is view name
    * path              URL related path generation from parents
    * param_managed     whether manage parameters before dispatching to handler
    * param_fields      parameter's name and field pairs definition,
                        this attribute will generate all parents' param_fields
    * param_dependency  dependency between parameters,
                        format as (name, ((field, [(name, field), ...]), ...))
    * form              form class used to manage parameters
    * decorators        view's decorators generate with the nearest-first logic,
                        this attribute will generate all parents' decorators
    """

    def __init__(self, options=None, parent=None):
        self.view = None
        self.children = []

        self.name = getattr(options, 'name', '')
        self.decorators = list(reversed(getattr(options, 'decorators', ())))

        if parent:
            p_opts = parent._meta
            self.parent = parent
            self.path = getattr(options, 'path', None)
            self.form = getattr(options, 'form', p_opts.form)
            self.param_managed = getattr(options, 'param_managed', p_opts.param_managed)
            self.param_fields = p_opts.param_fields.copy()
            self.param_dependency = p_opts.param_dependency.copy()
            self.param_fields.update(getattr(options, 'param_fields', ()))
            self.param_dependency.update(getattr(options, 'param_dependency', ()))
            self.decorators.extend(p_opts.decorators)
        else:
            self.path = getattr(options, 'path', '/')
            self.parent = None
            self.form = getattr(options, 'form', forms.Form)
            self.param_fields = OrderedDict(getattr(options, 'param_fields', ()))
            self.param_dependency = OrderedDict(getattr(options, 'param_dependency', ()))
            self.param_managed = getattr(options, 'param_managed', True)

    def gen_param_form(self, cls):
        form_attrs = dict(self.param_fields)
        form_attrs['__module__'] = cls.__module__
        cls.param_form = type(self.form)(cls.__name__ + 'Form',  (self.form, ), form_attrs)

    def contribute_to_class(self, cls, name):
        self.view = cls
        try:
            func_extend_param_fields = getattr(cls, 'get_extend_param_fields')
            if callable(func_extend_param_fields):
                self.param_fields.update(func_extend_param_fields())
        except Exception:
            pass
        if not self.name:
            self.name = self.view.__name__
        if self.path is None:
            parent_path = self.parent._meta.path
            if not parent_path.endswith("/"):
                parent_path += "/"
            self.path = urljoin(parent_path, _camel2path(self.name))
        if self.parent:
            self.parent._meta.children.append(cls)

        self.gen_param_form(cls)

        setattr(cls, name, self)

    def decorate_handler(self, handler):
        for decorator in self.decorators:
            handler = decorator(handler)
        return handler


class ViewMetaclass(type):
    """Metaclass of the view classes"""

    def __new__(mcs, name, bases, attrs):
        parents = [b for b in bases if isinstance(b, ViewMetaclass)]
        assert len(parents) <= 1

        __name__ = attrs.pop('__name__', name)

        new_cls = super(ViewMetaclass, mcs).__new__(mcs, name, bases, attrs)

        option_class = getattr(new_cls, 'option_class', ViewOptions)

        if parents:
            opts = option_class(attrs.pop('Meta', None), parents[0])
        else:
            opts = option_class(attrs.pop('Meta', None))

        opts.contribute_to_class(new_cls, '_meta')

        new_cls.__name__ = __name__

        return new_cls


class ViewMixin(DjangoView, metaclass=ViewMetaclass):
    """Base view generated from django view"""

    option_class = ViewOptions

    def init_params(self, request, *args, **kwargs):
        request.params = Param(self, request, kwargs)

    def dispatch(self, request, *args, **kwargs):
        if request.method.lower() in self.http_method_names:
            handler = getattr(self, request.method.lower(), self.http_method_not_allowed)
        else:
            handler = self.http_method_not_allowed
        try:
            if handler is not self.http_method_not_allowed:
                self.init_params(request, *args, **kwargs)
            return handler(request, *args, **kwargs)
        except Exception as exc:
            return self.handle_exception(exc)

    def handle_exception(self, exc):
        if hasattr(exc, 'error_dict'):
            return http.HttpResponseBadRequest(exc.error_dict.as_text())
        else:
            return http.HttpResponseBadRequest(exc)

    @classmethod
    def as_view(cls, *args, **kwargs):
        view = super().as_view(*args, **kwargs)
        return cls._meta.decorate_handler(view)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import json
import logging
from collections import OrderedDict

from django.conf import settings
from django.forms import forms
from django.http import HttpResponse
from django.template.loader import render_to_string
from django.utils.datastructures import MultiValueDict
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _, gettext_lazy
from rest_framework import fields, serializers
from rest_framework.fields import empty
from rest_framework.views import APIView

from cool.settings import cool_settings
from cool.views.error_code import ErrorCode
from cool.views.exceptions import CoolAPIException
from cool.views.options import ViewMetaclass, ViewOptions
from cool.views.param import Param
from cool.views.response import ResponseData

logger = logging.getLogger('cool.views')


class ParamSerializer(serializers.Serializer):
    def __init__(self, instance=None, data=empty, files=None, **kwargs):
        d = MultiValueDict()
        if data is not empty:
            d.update(data)
        if files:
            d.update(files)
        if not d:
            request = kwargs.get('context', dict()).get('request', None)
            view = kwargs.get('context', dict()).get('view', None)
            if request is not None:
                d.update(request.GET)
                d.update(request.POST)
                d.update(request.FILES)
            if view is not None and hasattr(view, 'kwargs'):
                d.update(view.kwargs)
        super().__init__(instance, d, **kwargs)
        self.is_valid()

    @property
    def cleaned_data(self):
        self.is_valid()
        data = self.validated_data.copy()
        for key, field in self.fields.items():
            if key not in data:
                data[key] = getattr(field, 'default', empty)
        return data

    def update(self, instance, validated_data):
        raise RuntimeError()

    def create(self, validated_data):
        raise RuntimeError()


class APIViewOptions(ViewOptions):
    """APIView options class

    Extend options:
    * wrappers          view's wrappers generate with the nearest-first logic,
                        this attribute will generate all parents' wrappers
    """

    def __init__(self, options=None, parent=None):
        super().__init__(options, parent)
        if not hasattr(options, 'form') and self.form is forms.Form:
            self.form = ParamSerializer
        self.wrappers = list(reversed(getattr(options, 'wrappers', ())))
        if self.parent:
            self.wrappers.extend(parent._meta.wrappers)

    def wrap_view(self, view):
        for wrapper in self.wrappers:
            view = wrapper(view)
        return view

    def gen_param_form(self, cls):
        form_attrs = dict(self.param_fields)

        class Meta:
            fields = list(form_attrs.keys())

        form_attrs['__module__'] = cls.__module__
        form_attrs['Meta'] = Meta
        cls.param_form = type(self.form)(cls.__name__ + 'ParamSerializer',  (self.form, ), form_attrs)
        cls.serializer_class = cls.param_form


class PageMixin:
    PAGE_SIZE_MAX = 200

    @classmethod
    def get_extend_param_fields(cls):
        return (
            (
                'page', fields.IntegerField(
                    label=gettext_lazy('Page number'),
                    default=1,
                    help_text=gettext_lazy('Start with %(start)s') % {'start': 1}
                )
            ),
            ('page_size', fields.IntegerField(label=gettext_lazy('Page size'), default=100)),
        )

    @classmethod
    def response_info_data(cls):
        return {
            'page_size': _('Page size'),
            'list': [super().response_info_data()],
            'page': _('Page number'),
            'total_page': _('Total page'),
            'total_data': _('Total data')
        }

    def get_page_context(self, request, queryset, serializer_cls):
        page_size = request.params.page_size
        if page_size <= 0 or page_size > self.PAGE_SIZE_MAX:
            raise CoolAPIException(ErrorCode.ERR_PAGE_SIZE_ERROR)
        total_data = queryset.count()
        total_page = (total_data + page_size - 1) // page_size
        page = request.params.page
        data = []
        if total_data > 0 and 1 <= page <= total_page:
            start = (page - 1) * page_size
            data = serializer_cls(queryset[start:start + page_size], request=request, many=True).data

        return {'page_size': page_size, 'list': data, 'page': page, 'total_page': total_page, 'total_data': total_data}


class CoolBFFAPIView(APIView, metaclass=ViewMetaclass):
    """
    Backend For Frontend APIView
    """

    option_class = APIViewOptions
    SYSTEM_ERROR_STATUS_CODE = cool_settings.API_SYSTEM_ERROR_STATUS_CODE
    PARAM_ERROR_STATUS_CODE = cool_settings.API_PARAM_ERROR_STATUS_CODE
    SUCCESS_WITH_CODE = cool_settings.API_SUCCESS_WITH_CODE
    SHOW_PARAM_ERROR_INFO = cool_settings.API_SHOW_PARAM_ERROR_INFO
    description_template_name = 'cool/views/api_description.html'

    response_info_serializer_class = None
    response_many = False

    def initialize_request(self, request, *args, **kwargs):
        _ = request.body
        return super().initialize_request(request, *args, **kwargs)

    def get_view_description(self, html=False):
        if not html or not self.description_template_name:
            return super().get_view_description(html)
        view_info = self.get_view_info()
        return mark_safe(render_to_string(self.description_template_name, view_info))

    @classmethod
    def get_view_info(cls):
        request_info = cls.request_info_data()
        response_info = ResponseData(cls.response_info_data()).get_response_data()
        return {
            'request_info': request_info,
            'response_info': response_info,
            'response_info_format': json.dumps(response_info, ensure_ascii=False, indent=4)
        }

    @classmethod
    def response_info_data(cls):
        if cls.response_info_serializer_class is not None:
            from cool.views.utils import get_serializer_info
            return get_serializer_info(cls.response_info_serializer_class(), cls.response_many)
        return None

    @classmethod
    def request_info_data(cls):
        from cool.views.utils import get_field_info
        ret = OrderedDict()
        serializer_class = getattr(cls, 'serializer_class', None)
        if serializer_class is not None:
            serializer = serializer_class()
            for key, field in serializer.fields.items():
                ret[key] = get_field_info(field)
        return ret

    def init_params(self, request, *args, **kwargs):
        data = MultiValueDict()
        data.update(request.GET)
        data.update(request.POST)
        data.update(kwargs)
        request.params = Param(self, request, data, request.FILES)

    @classmethod
    def get_response(cls, context):
        if isinstance(context, HttpResponse):
            return context
        elif not isinstance(context, ResponseData):
            context = ResponseData(context)
        return context.get_response()

    def check_api_permissions(self, request, *args, **kwargs):
        pass

    def view(self, request, *args, **kwargs):
        self.init_params(request, *args, **kwargs)
        self.check_api_permissions(request, *args, **kwargs)
        context = self.get_context(request, *args, **kwargs)
        return self.get_response(context)

    def get(self, request, *args, **kwargs):
        return self.view(request, *args, **kwargs)

    post = get

    def get_context(self, request, *args, **kwargs):
        raise NotImplementedError

    @classmethod
    def get_param_error_info(cls, exc):
        data = dict()
        if cls.SHOW_PARAM_ERROR_INFO:
            if hasattr(exc, 'error_dict_obj'):
                data['errors'] = exc.error_dict_obj
            else:
                data['desc'] = force_text(exc)
        return data

    def get_uncaught_exception_response(self, exc, context):
        if settings.DEBUG:
            return None
        logger.error("uncaught_exception", exc_info=exc, extra={'request': self.request})
        return self.get_response(ResponseData(None, ErrorCode.ERROR_SYSTEM, status_code=self.SYSTEM_ERROR_STATUS_CODE))

    def get_exception_handler(self):
        super_exception_handler = super().get_exception_handler()

        def get_exception_handler(exc, context):
            ret = super_exception_handler(exc, context)
            if ret is None:
                ret = self.get_uncaught_exception_response(exc, context)
            return ret
        return get_exception_handler

    def handle_exception(self, exc):
        if isinstance(exc, forms.ValidationError):
            exc = CoolAPIException(
                ErrorCode.ERROR_BAD_PARAMETER,
                data=self.get_param_error_info(exc),
                status_code=self.PARAM_ERROR_STATUS_CODE
            )
        if isinstance(exc, CoolAPIException):
            return self.get_response(exc.response_data)
        return super().handle_exception(exc)



# encoding: utf-8
from django.urls import path

from cool.admin.views import CoolAutocompleteJsonView

urlpatterns = [
    path('admin/autocomplete/', CoolAutocompleteJsonView.as_view(), name="cool_admin_autocomplete"),
]



# encoding: utf-8

from functools import reduce, wraps

import django
from django import forms
from django.contrib import admin
from django.contrib.admin.utils import flatten_fieldsets
from django.contrib.admin.widgets import AdminFileWidget
from django.contrib.auth import get_permission_codename
from django.core.exceptions import (
    FieldDoesNotExist, PermissionDenied, ValidationError,
)
from django.core.validators import EMPTY_VALUES
from django.db import models
from django.db.models.constants import LOOKUP_SEP
from django.db.models.fields.related import RelatedField
from django.forms.models import (
    BaseInlineFormSet, BaseModelFormSet, modelform_factory,
)
from django.urls import NoReverseMatch, reverse
from django.utils.encoding import force_str
from django.utils.http import urlencode
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from cool.admin import widgets
from cool.settings import cool_settings


def extend_admincls(*admin_classes):
    assert admin_classes
    return type('_Admin', admin_classes, {})


def get_widget(field):
    if isinstance(field, models.ImageField):
        return widgets.ImageWidget
    return widgets.TagWidget


def get_field_widget(field):
    if isinstance(field, models.ImageField):
        return widgets.ImageFieldWidget
    return widgets.FieldWidget


def _lookup_field(name, obj):
    splices = name.split(LOOKUP_SEP)
    field_name = splices[-1]
    for gen in splices[:-1]:
        obj = getattr(obj, gen)

    field = obj._meta.get_field(field_name)
    if isinstance(field_name, RelatedField):
        label = field.remote_field.field.verbose_name
    else:
        label = field.verbose_name

    return field, label, getattr(obj, field.attname)


def format_field(verbose, field, options=None, **kwargs):
    """
    Format field with given widgets or default

    Example:
        class MyAdminClass(models.ModelAdmin):
            list_display = [format_field(
                verbose='名字', field='username', options={
                    'style': 'color:red'
                }), ...]
    """
    name = field
    widget_opts = options or {}

    def _format_field(obj):
        field, label, value = _lookup_field(name, obj)
        attrs = widget_opts.copy()
        widget_class = attrs.pop('widget_class', None)
        if widget_class is None:
            widget_class = get_widget(field)
        widget = widget_class(attrs)
        return mark_safe(widget.render(label, value, {}))

    _format_field.short_description = verbose
    for att, val in kwargs.items():
        setattr(_format_field, att, val)
    return _format_field


def collapse_fields(verbose, fields, options=None, **kwargs):
    """
    Collapse_fields into one field in changelist_view in modeladmin site

    Example:
        class MyAdminClass(models.ModelAdmin):
            list_display = [collapse_fields(
                verbose='名字', fields=('username', 'name'), options={
                    'username':{'style': 'color:red'}
                }), ...]
    """
    widget_map = {}
    for name in fields:
        if options:
            widget_kwargs = options.get(name, {})
            widget_class = widget_kwargs.pop('widget_class', None)
            widget_map[name] = {
                'widget_class': widget_class,
                'attrs': widget_kwargs,
            }
        else:
            widget_map[name] = {}

    def format_fields(obj):
        html = []
        for name in fields:
            field, label, value = _lookup_field(name, obj)
            widget_class = widget_map[name].get('widget_class', None) or get_field_widget(field)
            widget = widget_class(widget_map[name].get('attrs', {}))
            html.append(widget.render(name, (label, value), None))
        return mark_safe(''.join(html))

    format_fields.short_description = verbose
    for att, val in kwargs.items():
        setattr(format_fields, att, val)
    return format_fields


def get_related_model_fields(model, rel, is_foreign_key):
    """
    通过model下rel对象获取相关字段或关联属性
    """
    # 多对多关联的REL对象本身不区分关系前后
    # 相关代理类做同样处理
    if is_foreign_key and rel.field.model._meta.concrete_model == model._meta.concrete_model:
        return rel.field, rel.get_related_field()
    return rel.get_related_field(), rel.field


class FormSetMixin:
    """
    Add validation to ensure POST data is up to date
    """
    def _existing_object(self, pk):
        if not hasattr(self, '_object_dict'):
            queryset = self.get_queryset()
            if self.data:  # 提交数据时
                pk_field = self.model._meta.pk
                pks = []
                for i in range(self.total_form_count()):
                    prefix = self.add_prefix(i)
                    pk_val = self.data.get('%s-%s' % (prefix, pk_field.name))
                    if pk_val is not None:
                        pks.append(pk_val)
                queryset = queryset.filter(pk__in=pks)
            self._object_dict = {o.pk: o for o in queryset}
        obj = self._object_dict.get(pk)
        if obj is None:
            obj = self.get_queryset().filter(pk=pk).first()
            self._object_dict[pk] = obj
        return obj

    def clean(self):
        super().clean()
        self.validate_queryset()

    def validate_queryset(self):
        pk = self.model._meta.pk
        to_python = pk.to_python
        rel = pk.remote_field
        while rel:
            related_field = rel.get_related_field()
            to_python = related_field.to_python
            rel = related_field.remote_field

        updated = False
        for form in self.forms:
            pk_data = form[pk.name].data
            if (pk_data not in EMPTY_VALUES
                    and form.instance.pk != to_python(pk_data)):
                updated = True
                break
        if updated:
            new_formset = self.__class__(
                save_as_new=self.save_as_new,
                prefix=self.prefix,
                queryset=self.queryset,
            )
            self.forms = new_formset.forms
            raise ValidationError('页面数据已经更新，请修改后重新保存')


class StrictModelFormSet(FormSetMixin, BaseModelFormSet):
    pass


class StrictInlineFormSet(FormSetMixin, BaseInlineFormSet):
    pass


class AdminImageWidget(AdminFileWidget):
    def render(self, name, value, attrs=None, renderer=None):
        output = []
        if value and getattr(value, "url", None):
            image_url = value.url
            file_name = str(value)
            output.append(' <a href="%s" target="_blank">'
                          '<img src="%s" alt="%s" style="max-width: 500px; max-height: 1000px" /></a>' %
                          (image_url, image_url, file_name))
        output.append(super().render(name, value, attrs, renderer))
        return mark_safe(''.join(output))


class BaseModelAdmin(admin.ModelAdmin):
    """
    default Admin class used by proxy|real models
    """

    # remove "__str__"
    list_display = []

    # Extend options to manage site
    # extend field exclude RelatedField and PrimaryKey fields into list_display
    empty_value_display = _('[None]')
    with_related_items = True
    extend_normal_fields = True
    extend_related_fields = False
    exclude_list_display = []
    heads = ['id', ]
    tails = []
    # manage Add/Change view
    addable = True
    changeable = True
    deletable = True
    # manage Change view
    change_view_readonly_fields = []
    changeable_fields = forms.ALL_FIELDS

    formset = StrictModelFormSet

    def __getattr__(self, attr):
        if ('__' in attr
                and not attr.startswith('_')
                and not attr.endswith('_boolean')
                and not attr.endswith('_short_description')):

            def dyn_lookup(instance):
                # traverse all __ lookups
                return reduce(lambda parent, child: getattr(parent, child), attr.split('__'), instance)

            # get admin_order_field, boolean and short_description
            dyn_lookup.admin_order_field = attr
            dyn_lookup.boolean = getattr(self, '{}_boolean'.format(attr), False)
            dyn_lookup.short_description = getattr(
                self, '{}_short_description'.format(attr),
                attr.replace('_', ' ').capitalize()
            )

            return dyn_lookup

        # not dynamic lookup, default behaviour
        return self.__getattribute__(attr)

    def formfield_for_dbfield(self, db_field, **kwargs):
        if isinstance(db_field, models.ImageField):
            kwargs.pop("request", None)
            kwargs['widget'] = AdminImageWidget
            return db_field.formfield(**kwargs)
        return super().formfield_for_dbfield(db_field, **kwargs)

    def get_changeable_fields(self, request, obj=None):
        if not self.changeable:
            return ()
        if self.changeable_fields == forms.ALL_FIELDS:
            return None
        elif self.changeable_fields is None:
            return ()

        return self.changeable_fields

    def has_delete_permission(self, request, obj=None):
        return self.deletable and super().has_delete_permission(request, obj)

    def has_change_permission(self, request, obj=None):
        return self.changeable and super().has_change_permission(request, obj)

    def has_add_permission(self, request):
        return self.addable and super().has_add_permission(request)

    def get_user_queryset(self, queryset):
        return queryset

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        db = kwargs.get('using')
        if ('widget' not in kwargs
                and cool_settings.FOREIGNKEY_FIELD_USE_AUTOCOMPLETE
                and hasattr(db_field.remote_field.model, 'get_search_fields')
                and db_field.name not in [*self.raw_id_fields, *self.radio_fields]):
            kwargs['widget'] = widgets.CoolAutocompleteSelect(db_field.remote_field, self.admin_site, using=db)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        db = kwargs.get('using')
        if ('widget' not in kwargs
                and cool_settings.MANYTOMANY_FIELD_USE_AUTOCOMPLETE
                and hasattr(db_field.remote_field.model, 'get_search_fields')
                and db_field.name not in [*self.raw_id_fields, *self.filter_vertical, *self.filter_horizontal]):
            kwargs['widget'] = widgets.CoolAutocompleteSelectMultiple(db_field.remote_field, self.admin_site, using=db)
        if django.VERSION >= (3, 1) or 'widget' not in kwargs:
            return super().formfield_for_manytomany(db_field, request, **kwargs)
        else:
            if not db_field.remote_field.through._meta.auto_created:
                return None
            if 'queryset' not in kwargs:
                queryset = self.get_field_queryset(db, db_field, request)
                if queryset is not None:
                    kwargs['queryset'] = queryset

            form_field = db_field.formfield(**kwargs)
            return form_field

    def get_search_fields(self, request):
        search_fields = super().get_search_fields(request)
        get_search_fields_func = getattr(self.opts.model, 'get_search_fields', None)
        if get_search_fields_func and callable(get_search_fields_func):
            return tuple(set(get_search_fields_func()) | set(search_fields))
        return ()

    def get_form(self, request, obj=None, **kwargs):
        if 'fields' in kwargs:
            fields = kwargs.get('fields')
        else:
            # 'get_fieldsets' would call 'get_form' again with kwargs: field=None
            fields = flatten_fieldsets(self.get_fieldsets(request, obj))

        # get a blank form to fetch all fields
        form = modelform_factory(self.model, kwargs.get('form', self.form), exclude=())

        if fields is None or fields is forms.ALL_FIELDS:
            fields = form.base_fields.keys()

        readonly_fields = self.get_readonly_fields(request, obj, fields=fields)
        readonly_fields_set = set(readonly_fields)

        if 'exclude' not in kwargs:
            if self.exclude is None:
                exclude = []
            else:
                exclude = list(self.exclude)
            exclude.extend(readonly_fields)

            to_exclude_fields = []
            for name in fields:
                field = form.base_fields.get(name, None)
                if field is None:
                    continue
                try:
                    val = field.prepare_value(getattr(obj, name, None))
                except Exception:
                    val = None
                if val is None:
                    continue
                elif name in readonly_fields_set:
                    continue
                elif isinstance(field, forms.ModelMultipleChoiceField):
                    relates = list(val.all().values_list('pk', flat=True))
                    if not relates:
                        continue
                    queryset = self.get_user_queryset(field.queryset)
                    key = field.to_field_name or 'pk'
                    if len(relates) != queryset.filter(**{'%s__in' % key: relates}).count():
                        to_exclude_fields.append(name)
                        readonly_fields_set.add(name)
                elif isinstance(field, forms.ModelChoiceField):
                    queryset = self.get_user_queryset(field.queryset)
                    key = field.to_field_name or 'pk'
                    if not queryset.filter(**{key: val}).exists():
                        to_exclude_fields.append(name)
                        readonly_fields_set.add(name)
                elif isinstance(field, forms.ChoiceField):
                    if field.choices and val not in dict(field.choices):
                        to_exclude_fields.append(name)
                        readonly_fields_set.add(name)

            if to_exclude_fields:
                exclude.extend(to_exclude_fields)
                readonly_fields.extend(to_exclude_fields)
            kwargs['exclude'] = exclude

        kwargs['fields'] = fields
        form = super().get_form(request, obj, **kwargs)
        # remove the custom fields from base_fields which is in 'exlcude' or not
        # in 'fields' to prevent from checking these fields
        exclude = form._meta.exclude or ()
        for name in form.declared_fields:
            if name in exclude or name not in fields:
                form.base_fields.pop(name, None)
        return form

    def get_fieldsets(self, request, obj=None):
        fieldsets = list(super().get_fieldsets(request, obj))
        is_add = obj is None
        if (is_add and self.has_add_permission(request)) or (not is_add and self.has_change_permission(request, obj)):
            return fieldsets
        else:
            list_display = self.get_list_display(request)
            valid_f_names = flatten_fieldsets(fieldsets)
            fields = [f for f in list_display if f in valid_f_names]
            return [(None, {'fields': fields})]

    def get_readonly_fields(self, request, obj=None, fields=None):
        is_add = obj is None
        readonly_fields = list(super().get_readonly_fields(request, obj))
        if (is_add and not self.has_add_permission(request)) or \
                (not is_add and not self.has_change_permission(request, obj)):
            return fields if fields is not None else self.get_list_display(request)

        readonly_fields_set = set(readonly_fields)
        if not is_add:
            for field in self.change_view_readonly_fields:
                if field not in readonly_fields_set:
                    readonly_fields_set.add(field)
                    readonly_fields.append(field)

            changeable_fields = self.get_changeable_fields(request, obj)
            if changeable_fields is not None:
                changeable_fields_set = set(changeable_fields)
                declared_fields = self.fieldsets
                if declared_fields:
                    declared_fields = flatten_fieldsets(declared_fields)
                else:
                    declared_fields = [f.name for f in self.opts._get_fields(reverse=False)]
                for field in declared_fields:
                    if field not in changeable_fields_set and field not in readonly_fields_set:
                        readonly_fields_set.add(field)
                        readonly_fields.append(field)
        return readonly_fields

    def gen_access_rels(self, request):
        if not hasattr(request, '_access_rels'):
            _access_rels = []

            def _add_access_rels(_rel, is_foreign_key):
                target_field, remote_field = get_related_model_fields(self.model, _rel, is_foreign_key)
                rel_opts = remote_field.model._meta
                view_perm = '%s.%s' % (rel_opts.app_label, get_permission_codename('change', rel_opts))
                change_perm = '%s.%s' % (rel_opts.app_label, get_permission_codename('change', rel_opts))
                if request.user.has_perm(view_perm) or request.user.has_perm(change_perm):
                    try:
                        uri = reverse('admin:%s_%s_changelist' % (rel_opts.app_label, rel_opts.model_name))
                    except NoReverseMatch:
                        pass
                    else:
                        _access_rels.append((uri, target_field, remote_field, is_foreign_key))

            for rel in self.opts.related_objects:
                #  + tuple(r.remote_field for r in self.opts.many_to_many):
                _add_access_rels(rel, False)
            for key, field in self.opts._forward_fields_map.items():
                if key != field.name:
                    continue
                if field.is_relation and (field.many_to_one or field.one_to_one) \
                        and hasattr(field.remote_field, 'model') and field.remote_field.model:
                    _add_access_rels(field.remote_field, True)
            setattr(request, '_access_rels', _access_rels)
        setattr(self, '_access_rels', request._access_rels)

    def get_list_display(self, request):
        """
        get all fields except PK and Relations if extend_normal_fields is True
        """

        list_display = list(super().get_list_display(request))
        if self.with_related_items:
            self.gen_access_rels(request)
        if not self.extend_normal_fields:
            if hasattr(request, '_access_rels'):
                list_display.append('get_all_relations')
            return list_display

        field_names = []
        heads = []
        middles = []
        tails = []

        def _get_field(field_name):
            if not field_name or not isinstance(field_name, str):
                return field_name
            try:
                field = self.model._meta.get_field(field_name)
                if field and isinstance(field, models.ImageField):
                    return format_field(field.verbose_name, field.name)
                else:
                    return field_name
            except FieldDoesNotExist:
                return field_name

        while list_display:
            name = list_display.pop(0)
            if name in self.tails:
                tails.append(_get_field(name))
            elif name in self.heads:
                heads.append(_get_field(name))
            else:
                middles.append(_get_field(name))

        for field in self.get_normal_fields():
            if field.name in heads or field.name in middles or field.name in tails:
                continue
            if field.name in self.exclude_list_display:
                pass
            elif field.name in self.tails:
                tails.append(_get_field(field.name))
            elif field.name in self.heads:
                heads.append(_get_field(field.name))
            else:
                middles.append(_get_field(field.name))
        field_names.extend(heads)
        field_names.extend(middles)
        field_names.extend(tails)
        if hasattr(request, '_access_rels'):
            field_names.append('get_all_relations')
        return field_names

    def get_normal_fields(self):
        fields = []
        exclude = self.exclude or ()
        for field in self.model._meta.concrete_fields:
            if field.name.startswith('_') or field.attname in exclude:
                continue
            if isinstance(field, RelatedField):
                if not (self.extend_related_fields and (field.many_to_one or field.one_to_one)):
                    continue
            if isinstance(field, models.FileField) and not isinstance(field, models.ImageField):
                continue
            fields.append(field)
        return fields

    def get_all_relations(self, obj):
        if not getattr(self, '_access_rels'):
            return ''
        html_list = [
            '<select onchange="window.open(this.value, \'_self\');">',
            '<option value="" selected="selected">------</option>'
        ]
        for uri, target_field, remote_field, is_foreign_key in self._access_rels:
            rel_opts = remote_field.model._meta
            value = getattr(obj, target_field.attname)
            value = force_str(value)
            params = {remote_field.name: value}
            url = '%s?%s' % (uri, urlencode(params))
            if is_foreign_key:
                html_list.append('<option value="%s">%s</option>' % (url, target_field.verbose_name))
            else:
                html_list.append(
                    '<option value="%s">%s-%s</option>' % (url, rel_opts.verbose_name, remote_field.verbose_name)
                )
        html_list.append('</select>')
        return mark_safe(''.join(html_list))

    get_all_relations.short_description = _("Related items")


def check_perms(*perms):
    """
    Returns True if the given request has permissions to manage an object.
    """

    def inner(func):
        @wraps(func)
        def wrapper(admin, request, *args, **kwargs):
            opts = admin.opts
            for perm in perms:
                codename = get_permission_codename(perm, opts)
                if not request.user.has_perm("%s.%s" % (opts.app_label, codename)):
                    raise PermissionDenied
            return func(admin, request, *args, **kwargs)

        return wrapper

    return inner


def site_register(model_or_iterable, admin_class=None, site=None, **options):
    if site is None:
        site = admin.site
    if admin_class is None:
        admin_class = BaseModelAdmin
    site.register(model_or_iterable, admin_class, **options)



# encoding: utf-8
from cool.settings import cool_settings
from cool.views.error_code import ErrorCode
from cool.views.response import ResponseData


class CoolAPIException(Exception):

    def __init__(
        self, code, data=None, *, message=None,
        status_code=cool_settings.API_EXCEPTION_DEFAULT_STATUS_CODE
    ):
        self.response_data = ResponseData(data, code=code, message=message, status_code=status_code)
        super().__init__(self.response_data.message)


class CoolPermissionAPIException(CoolAPIException):
    def __init__(self, *, code=ErrorCode.ERROR_PERMISSION, status_code=403, **kwargs):
        super().__init__(code=code, status_code=status_code, **kwargs)



# encoding: utf-8

from rest_framework import serializers
from rest_framework.fields import empty


class BaseSerializer(serializers.ModelSerializer):

    def __init__(self, instance=None, data=empty, **kwargs):
        if data is None:
            data = empty
        request = kwargs.pop("request", None)
        if request is not None:
            kwargs.setdefault("context", {}).update({"request": request})
        super().__init__(instance, data, **kwargs)
        root = self.root
        if root is not None:
            root._context = self._context
        for field in self._readable_fields:
            if isinstance(field, BaseSerializer) and not hasattr(field, '_context'):
                field._context = self._context

    def __new__(cls, *args, **kwargs):
        # We override this method in order to auto magically create
        # `ListSerializer` classes instead when `many=True` is set.
        request = kwargs.pop("request", None)
        if "context" not in kwargs and request is not None:
            kwargs["context"] = {"request": request}
        return super().__new__(cls, *args, **kwargs)

    def build_property_field(self, field_name, model_class):
        field_class, field_kwargs = super().build_property_field(field_name, model_class)
        label = getattr(model_class, field_name).__doc__
        if label:
            label = label.strip()
        if label:
            field_kwargs.setdefault('help_text' if '\n' in label else 'label', label)
        return field_class, field_kwargs

    @property
    def request(self):
        return self.context.get("request", None)



# encoding: utf-8
import re
from collections import OrderedDict
from importlib import import_module

from django.conf import settings
from django.core.validators import BaseValidator, ProhibitNullCharactersValidator
from django.db import models
from django.shortcuts import render
from django.template.loader import render_to_string
from django.utils.translation import gettext as _
from rest_framework import fields, validators
from rest_framework.fields import empty
from rest_framework.serializers import ModelSerializer
from rest_framework.utils import model_meta

from cool.views.error_code import ErrorCode
from cool.views.view import CoolBFFAPIView


def get_rest_field_from_model_field(model, model_field, **kwargs):
    if isinstance(model_field, models.Field):
        model_field = model_field.name
    s = ModelSerializer()
    field_class, field_kwargs = s.build_field(model_field, model_meta.get_field_info(model), model, 0)
    field_kwargs.pop('read_only', None)
    gen_validators = field_kwargs.pop('validators', None)
    if gen_validators:
        gen_validators = list(filter(
            lambda x: not isinstance(x, (
                validators.UniqueValidator, validators.BaseUniqueForValidator, ProhibitNullCharactersValidator
            )),
            gen_validators
        ))
        if gen_validators:
            field_kwargs['validators'] = gen_validators
    field_kwargs = s.include_extra_kwargs(field_kwargs, kwargs)
    return field_class(**field_kwargs)


def get_field_info(field):
    field_type = field.__class__.__name__
    if field_type.endswith("Field") and field_type != 'Field':
        field_type = field_type[:-5]
    info = {
        'label': str(field.label),
        'type': field_type,
        'default': field.default,
        'default_format': '' if field.default is empty else field.default,
        'required': field.required,
        'required_format': _('Yes') if field.required is True else _('No'),
        'help_text': field.help_text,
        'extend_info': OrderedDict()
    }
    field_validators = [field]
    field_validators.extend(getattr(field, 'validators', list()))
    validator_keys = ['max_value', 'min_value', 'max_length', 'min_length', 'max_digits', 'max_decimal_places',
                      'sep', 'child', 'choices', 'regex', 'allowed_extensions']
    for validator in field_validators:
        for k in validator_keys:
            v = getattr(validator, k, None)
            if v is not None:
                if isinstance(v, re.Pattern):
                    v = v.pattern
                info['extend_info'].setdefault(k, list()).append(v)
        if isinstance(validator, BaseValidator):
            info['extend_info'].setdefault(validator.code, list()).append(validator.limit_value)
    for k in ['max_value', 'max_length', 'max_digits', 'max_decimal_places']:
        if k in info['extend_info']:
            info['extend_info'][k] = min(info['extend_info'][k])
    for k in ['min_value', 'min_length']:
        if k in info['extend_info']:
            info['extend_info'][k] = max(info['extend_info'][k])
    for k in info['extend_info']:
        if not isinstance(info['extend_info'][k], list):
            continue
        if len(info['extend_info'][k]) == 1:
            info['extend_info'][k] = info['extend_info'][k].pop()
    if 'choices' in info['extend_info']:
        info['extend_info'].pop('max_value', None)
        info['extend_info'].pop('min_value', None)

    def _format_value(_value):
        if isinstance(_value, list):
            return ",".join(map(lambda x: _format_value(x) if isinstance(x, dict) else x, _value))
        elif isinstance(_value, dict):
            return ",".join(["%s:%s" % (_k, _v) for _k, _v in _value.items()])
        else:
            return _value

    info['extend_info_format'] = "; ".join(["%s:%s" % (k, _format_value(v)) for k, v in info['extend_info'].items()])
    return info


def get_list_info(serializer_obj):
    child = serializer_obj.child
    if hasattr(child, 'fields'):
        return get_serializer_info(child, force_many=True)
    return [str(serializer_obj.label)]


def get_serializer_info(serializer_obj, force_many=False):
    ret = dict()
    for field_name, field in serializer_obj.fields.items():
        if hasattr(field, 'fields'):
            ret[field_name] = get_serializer_info(field)
        elif hasattr(field, 'child'):
            ret[field_name] = get_list_info(field)
        elif hasattr(field, 'child_relation'):
            ret[field_name] = [str(field.child_relation.label)]
        else:
            ret[field_name] = str(field.label)
    return [ret] if force_many else ret


def get_url(head, urlpattern):
    url = getattr(urlpattern, 'pattern', urlpattern).regex.pattern
    ret = head + url.replace('\\', '').rstrip("$?").lstrip('^')
    return ret.replace('//', '/')


def get_view_list(urlpattern=None, head='/', base_view=CoolBFFAPIView):
    ret = []
    if urlpattern is None:
        rooturl = import_module(settings.ROOT_URLCONF)
        for urlpattern in rooturl.urlpatterns:
            ret += get_view_list(urlpattern, get_url(head, urlpattern), base_view=base_view)
        return ret
    view_class = urlpattern
    for sub in ('callback', 'view_class'):
        view_class = getattr(view_class, sub, None)
        if view_class is None:
            break
    if view_class is not None and issubclass(view_class, base_view):
        retdict = dict()
        retdict['view_class'] = view_class
        retdict['params'] = view_class._meta.param_fields
        retdict['name'] = view_class().get_view_name()
        retdict['url'] = head.replace('//', '/').rstrip('/')
        ret.append(retdict)

    if hasattr(urlpattern, 'url_patterns'):
        for pattern in urlpattern.url_patterns:
            ret += get_view_list(pattern, get_url(head, pattern), base_view=base_view)

    return ret


def get_api_info(base_view=CoolBFFAPIView, exclude_params=(), exclude_base_view_params=True, exclude_views=()):
    assert issubclass(base_view, CoolBFFAPIView)
    exclude_params = set(exclude_params)
    if exclude_base_view_params:
        opt = getattr(base_view, '_meta', None)
        param_fields = getattr(opt, 'param_fields', dict())
        exclude_params.update(param_fields.keys())

    error_codes = ErrorCode.get_desc_dict()

    apis = list()
    for v in get_view_list(base_view=base_view):
        if issubclass(v['view_class'], exclude_views):
            continue
        has_file = False
        post = False
        length = 0
        no_len_count = 0
        for param, field in v['params'].items():
            if isinstance(field, fields.FileField):
                has_file = True
                post = True
            if param in exclude_params:
                continue
            if param in ('pass', 'password'):
                post = True
            if isinstance(field, fields.CharField):
                if field.max_length is None:
                    no_len_count += 1
                else:
                    length += field.max_length

        if no_len_count > 3 or length > 200:
            post = True

        apis.append({
            'name': v['name'],
            'url': v['url'],
            'ul_name': v['url'].replace('/', '_').strip('_'),
            'info': v['view_class'].get_view_info(),
            'suggest_method': 'POST' if post else 'GET',
            'content_type': 'multipart/form-data' if has_file else 'application/x-www-form-urlencoded',
        })
    return {
        'exclude_params': list(exclude_params),
        'error_codes': error_codes,
        'apis': apis
    }


def get_api_doc(
    request=None,
    template_name='cool/views/api_doc.md',
    base_view=CoolBFFAPIView,
    exclude_params=(),
    exclude_base_view_params=True,
    exclude_views=()
):
    api_info = get_api_info(base_view, exclude_params, exclude_base_view_params, exclude_views)
    api_info['server'] = request.build_absolute_uri("/")[:-1] if request is not None else '/'
    return render_to_string(template_name, api_info, request)


def get_api_doc_html(request, *args, **kwargs):
    md_template_name = kwargs.get('md_template_name', 'cool/views/api_doc.md')
    base_view = kwargs.get('base_view', CoolBFFAPIView)
    exclude_params = kwargs.get('exclude_params', ())
    exclude_base_view_params = kwargs.get('exclude_base_view_params', True)
    exclude_views = kwargs.get('exclude_views', ())
    title = kwargs.get('title', _('Api Document'))
    md = get_api_doc(
        request=request,
        template_name=md_template_name,
        base_view=base_view,
        exclude_params=exclude_params,
        exclude_base_view_params=exclude_base_view_params,
        exclude_views=exclude_views
    )
    import markdown
    html = markdown.markdown(md, extensions=[
        'markdown.extensions.toc',
        'markdown.extensions.fenced_code',
        'markdown.extensions.tables'
    ])
    md_style_template_name = kwargs.get('md_style_template_name', 'cool/views/markdown.html')
    return render(request, md_style_template_name, context={'html': html, 'title': title})



# encoding: utf-8
import random

from django import forms
from django.contrib import admin
from django.contrib.admin import widgets
from django.contrib.admin.utils import get_model_from_relation
from django.contrib.admin.views.main import ERROR_FLAG, PAGE_VAR
from django.forms.widgets import MEDIA_TYPES, Media
from django.utils.http import urlencode
from django.utils.translation import gettext as _

from cool.admin.widgets import (
    CoolAutocompleteSelect, DateRangeFilterWidget, NumberRangeFilterWidget,
)


class AutocompleteSelect(widgets.AutocompleteSelect):
    def __init__(self, rel, admin_site, attrs=None, choices=(), using=None, custom_url=None):
        self.custom_url = custom_url
        super().__init__(rel, admin_site, attrs, choices, using)

    def get_url(self):
        return self.custom_url if self.custom_url else super().get_url()


class WidgetFilterMixin:
    template = 'cool/admin/widget_filter.html'
    auto_commit = False
    widget = None
    widget_attrs = {}

    def __init__(self, field, request, params, model, model_admin, field_path, *args, **kwargs):
        super().__init__(field, request, params, model, model_admin, field_path, *args, **kwargs)
        self.field_path = field_path
        self.params = dict(request.GET.items())
        if PAGE_VAR in self.params:
            del self.params[PAGE_VAR]
        if ERROR_FLAG in self.params:
            del self.params[ERROR_FLAG]
        self.init_widget(field, request, params, model, model_admin, field_path, *args, **kwargs)
        if self.widget:
            self._add_media(model_admin, self.widget)

    def _add_media(self, model_admin, widget):
        if not getattr(widget, 'media', None):
            return

        model_admin_cls = model_admin.__class__
        if not hasattr(model_admin_cls, 'Media'):
            model_admin_cls.Media = Media

        def _get_media(obj):
            return Media(media=getattr(obj, 'Media', None))
        media = _get_media(model_admin) + widget.media + _get_media(self)
        for name in MEDIA_TYPES:
            setattr(model_admin.Media, name, getattr(media, "_" + name))

    def init_widget(self, field, request, params, model, model_admin, field_path, *args, **kwargs):
        self.widget = self.widget(attrs=self.get_widget_attrs(self.get_param_name(), params, self.widget_attrs))

    def get_widget_context(self):
        if not self.widget:
            return {}
        return self.widget.get_context(self.get_param_name(), self.params.get(self.get_param_name()), {})

    @classmethod
    def set_class(cls, widget_context):
        if 'attrs' not in widget_context:
            return
        attrs_class = widget_context['attrs'].get("class", None)
        if attrs_class:
            attrs_class += " admin-widget-filter"
        else:
            attrs_class = "admin-widget-filter"
        widget_context['attrs']['class'] = attrs_class

    @property
    def widget_render(self):
        if not self.widget:
            return ''
        context = self.get_widget_context()
        if 'widget' in context:
            self.set_class(context['widget'])
            if 'subwidgets' in context['widget'] and len(context['widget']['subwidgets']) == 2:
                self.set_class(context['widget']['subwidgets'][0])
                self.set_class(context['widget']['subwidgets'][1])
        return self.widget._render(self.widget.template_name, context, None)

    def get_param_name(self):
        raise NotImplementedError

    def get_widget_attrs(self, param_name, params, attrs=None):
        widget_attrs = attrs.copy() if attrs else {}
        widget_attrs['data-query-string-value'] = 'data-value-%s' % random.randint(100000000, 999999999)
        params = params.copy() if params else {}
        params[param_name] = widget_attrs['data-query-string-value']
        widget_attrs['data-query-string'] = "?" + urlencode(params)
        params.pop(param_name)
        widget_attrs['data-query-string-all'] = "?" + urlencode(params)
        return widget_attrs

    def choices(self, changelist):
        return ()

    def expected_parameters(self):
        return ()

    class Media:
        js = (
            'admin/js/jquery.init.js',
            'cool/admin/widget_filter.js'
        )


class RangeFilterMixin(WidgetFilterMixin):

    widget_attrs = {'style': 'width:100%'}

    def get_param1_name(self):
        return '%s__gte' % self.field_path

    def get_param2_name(self):
        return '%s__lte' % self.field_path

    def get_param_name(self):
        return self.field_path

    def init_widget(self, field, request, params, model, model_admin, field_path, *args, **kwargs):
        self.widget = self.widget(
            attrs=self.widget_attrs,
            sub1_attrs=self.get_widget_attrs(self.get_param1_name(), params),
            sub2_attrs=self.get_widget_attrs(self.get_param2_name(), params),
        )

    def get_widget_context(self):
        return self.widget.get_context(
            self.get_param_name(),
            [self.params.get(self.get_param1_name()), self.params.get(self.get_param2_name())],
            {}
        )


class DateRangeFieldFilter(RangeFilterMixin, admin.FieldListFilter):
    widget = DateRangeFilterWidget

    def get_param2_name(self):
        return '%s__date__lte' % self.field_path


class NumberRangeFieldFilter(RangeFilterMixin, admin.FieldListFilter):
    widget = NumberRangeFilterWidget


class AutocompleteFieldFilter(WidgetFilterMixin, admin.RelatedFieldListFilter):
    # def field_choices(self, field, request, model_admin):
    #     return ()
    widget_attrs = {'style': 'width: 100%'}

    def get_param_name(self):
        return self.lookup_kwarg

    def init_widget(self, field, *args, **kwargs):
        widget = CoolAutocompleteSelect(
            field.remote_field, attrs=self.get_widget_attrs(self.get_param_name(), self.params, self.widget_attrs)
        )
        other_model = get_model_from_relation(field)
        field = forms.ModelChoiceField(
            queryset=other_model.objects,
            widget=widget,
            required=False,
        )
        self.widget = field.widget

    def get_widget_attrs(self, param_name, params, attrs=None):
        widget_attrs = super().get_widget_attrs(param_name, params, attrs)
        widget_attrs['data-Placeholder'] = _("Please select %(title)s") % {'title': self.title}
        return widget_attrs


class SearchFilter(WidgetFilterMixin, admin.FieldListFilter):
    widget = widgets.AdminTextInputWidget
    widget_attrs = {'style': 'width: 100%;box-sizing: border-box;'}

    def get_param_name(self):
        return '%s__exact' % self.field_path


class ISearchFilter(SearchFilter):
    def get_param_name(self):
        return '%s__iexact' % self.field_path


class StartswithFilter(SearchFilter):
    def get_param_name(self):
        return '%s__startswith' % self.field_path


class IStartswithFilter(SearchFilter):
    def get_param_name(self):
        return '%s__istartswith' % self.field_path


class ContainsFilter(SearchFilter):
    def get_param_name(self):
        return '%s__contains' % self.field_path


class IContainsFilter(SearchFilter):
    def get_param_name(self):
        return '%s__icontains' % self.field_path



# encoding: utf-8

from django.db import models

from cool.core import cache


class ModelCache(cache.BaseCache):
    key_prefix = 'cool:model_cache'
    default_timeout = 600

    pk = cache.CacheItem()
    unique = cache.CacheItem()

    def _get_key(self, model_cls, field_key, field_name):
        assert field_key is not None
        assert issubclass(model_cls, models.Model)
        if field_name is None:
            return "%s:%s" % (model_cls._meta.db_table, field_key), self.pk
        filed = model_cls._meta.get_field(field_name)
        assert filed.unique
        return "%s:%s:%s" % (model_cls._meta.db_table, filed.name, field_key), self.unique

    def get(self, model_cls, field_key, field_name=None, *, ttl=None):
        key, item = self._get_key(model_cls, field_key, field_name)
        value = item.get(key)
        if value is None:
            if field_name is None:
                value = model_cls.objects.filter(pk=field_key).first()
            else:
                value = model_cls.objects.filter(**{field_name: field_key}).first()
            if value is not None:
                if ttl is None:
                    ttl = self.default_timeout
                item.set(key, value, ttl)
        return value

    def delete(self, model_cls, field_key, field_name=None):
        key, item = self._get_key(model_cls, field_key, field_name)
        return item.delete(key)


model_cache = ModelCache()



# encoding: utf-8

from urllib.parse import urlencode

from django import forms
from django.contrib.admin import widgets
from django.forms.utils import flatatt
from django.urls import reverse, reverse_lazy
from django.utils.html import format_html


def styles2python(str_style):
    dt = {}
    for css_item in str_style.strip().split(';'):
        css_item = css_item.strip()
        if not css_item:
            continue
        attr, val = css_item.split(':', 1)
        dt[attr.strip()] = val.strip()
    return dt


def python2styles(dt_style):
    return '; '.join([': '.join(css_pair) for css_pair in dt_style.items()])


class TagWidget(forms.Widget):
    tag = 'span'

    css = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.attrs['css'] = self._patch_inline_style(self.attrs) or {}
        if self.css:
            for att, val in self.css.items():
                self.attrs['css'].setdefault(att, val)

    def render(self, name, value, attrs=None, renderer=None):
        final_attrs = self.build_attrs(attrs)
        self.patch_inline_style(final_attrs)
        return format_html('<{tag} {attrs}>{value}</{tag}>', tag=self.tag, value=value, attrs=flatatt(final_attrs))

    def _patch_inline_style(self, attrs):
        self.tag = attrs.pop('tag', None) or self.tag
        css = attrs.pop('css', None) or self.css
        style = attrs.pop('style', None)
        style = style and styles2python(style)
        if style and css:
            style.update(css)
        elif style or css:
            style = style or css
        else:
            style = None
        return style

    def patch_inline_style(self, attrs):
        style = self._patch_inline_style(attrs)
        if style:
            attrs['style'] = python2styles(style)


class ImageWidget(TagWidget):
    tag = 'img'
    css = {
        'max-width': '80px',
        'max-height': '80px',
    }

    def render(self, name, value, attrs=None, renderer=None):
        final_attrs = self.build_attrs(attrs)
        self.patch_inline_style(final_attrs)
        if value:
            return format_html(
                '<a target="_blank" href="{url}"><{tag} {attrs} src="{url}" /></a>',
                tag=self.tag,
                url=value.url,
                attrs=flatatt(final_attrs)
            )
        else:
            return ''


class TagH4(TagWidget):
    tag = 'h4'
    css = {
        'background': 'transparent',
    }


class TagP(TagWidget):
    tag = 'p'


class FieldWidget(forms.MultiWidget):
    name_widget = TagH4
    value_widget = TagP

    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {}

        widgets = [self.name_widget(attrs), self.value_widget(attrs)]
        super().__init__(widgets, attrs)

    def format_output(self, rendered_widgets):
        return ''.join(rendered_widgets)

    def decompress(self, value):
        return value


class ImageFieldWidget(FieldWidget):
    value_widget = ImageWidget

    def format_output(self, rendered_widgets):
        return '<div>{0}</div>'.format(''.join(rendered_widgets))


class RangeFilterWidget(forms.MultiWidget):
    sub_widget = None
    template_name = 'cool/admin/range_filter_widget.html'

    def __init__(self, attrs=None, sub1_attrs=None, sub2_attrs=None):
        super().__init__((), attrs)
        self.widgets = [self.sub_widget(sub1_attrs), self.sub_widget(sub2_attrs)]

    def decompress(self, value):
        raise AssertionError("%s only use for filter" % self.__class__.__name__)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        style = styles2python(context['widget']['attrs'].setdefault('style', ''))
        style['display'] = 'flex'
        style['align-items'] = 'center'
        context['widget']['attrs']['style'] = python2styles(style)
        if 'widget' in context and 'subwidgets' in context['widget']:
            for subwidget in context['widget']['subwidgets']:
                style = styles2python(subwidget['attrs'].setdefault('style', ''))
                style['flex'] = "1"
                subwidget['attrs']['style'] = python2styles(style)
        return context


class NumberRangeFilterWidget(RangeFilterWidget):
    sub_widget = widgets.AdminIntegerFieldWidget


class AdminDateInRangeWidget(forms.DateInput):
    class Media:
        js = [
            'admin/js/jquery.init.js',
            reverse_lazy('admin:jsi18n'),
            'admin/js/calendar.js',
            'cool/admin/RangeDateShortcuts.js',
        ]
        css = {
            'all': ['admin/css/forms.css', ]
        }

    def __init__(self, attrs=None, format=None):
        attrs = {'class': 'vDateFieldInRange', 'autocomplete': 'off', 'size': '10', **(attrs or {})}
        super().__init__(attrs=attrs, format=format)


class DateRangeFilterWidget(RangeFilterWidget):
    sub_widget = AdminDateInRangeWidget


class CoolAutocompleteMixin(widgets.AutocompleteMixin):
    url = None

    def __init__(self, rel, admin_site=None, attrs=None, choices=(), using=None):
        widgets.AutocompleteMixin.__init__(self, rel, admin_site, attrs=attrs, choices=choices, using=using)
        self.to_field_name = getattr(self.rel, 'field_name', 'pk')
        if self.url is None:
            self.url = reverse('cool_admin_autocomplete')

    def get_limit_choices_to_params(self):
        limit_choices_to = self.rel.limit_choices_to
        if callable(limit_choices_to):
            limit_choices_to = limit_choices_to()
        return widgets.url_params_from_lookup_dict(limit_choices_to)

    def get_url(self):
        params = {
            'to_field_name': self.to_field_name,
            'app_label': self.rel.model._meta.app_label,
            'model_name': self.rel.model._meta.model_name,
        }
        limit_choices_to = self.get_limit_choices_to_params()
        if limit_choices_to:
            params['limit_choices_to'] = urlencode(limit_choices_to)
        return "%s?%s" % (self.url, urlencode(params))

    def optgroups(self, name, value, attr=None):
        default = (None, [], 0)
        groups = [default]
        has_selected = False
        selected_choices = {
            str(v) for v in value
            if str(v) not in self.choices.field.empty_values
        }
        if not self.is_required and not self.allow_multiple_selected:
            default[1].append(self.create_option(name, '', '', False, 0))

        choices = (
            (getattr(obj, self.to_field_name), self.choices.field.label_from_instance(obj))
            for obj in self.choices.queryset.using(self.db).filter(**{"%s__in" % self.to_field_name: selected_choices})
        )
        for option_value, option_label in choices:
            selected = (
                str(option_value) in value and
                (has_selected is False or self.allow_multiple_selected)
            )
            has_selected |= selected
            index = len(default[1])
            subgroup = default[1]
            subgroup.append(self.create_option(name, option_value, option_label, selected_choices, index))
        return groups


class CoolAutocompleteSelect(CoolAutocompleteMixin, widgets.AutocompleteSelect):
    pass


class CoolAutocompleteSelectMultiple(CoolAutocompleteMixin, widgets.AutocompleteSelectMultiple):
    pass



# encoding: utf-8

from django.conf import settings
from django.test.signals import setting_changed
from django.utils.module_loading import import_string

DEFAULTS = {
    # Admin
    'AUTOCOMPLETE_CHECK_PERM': True,
    'FILTER_USE_SELECT': True,
    'FOREIGNKEY_FIELD_USE_AUTOCOMPLETE': True,
    'MANYTOMANY_FIELD_USE_AUTOCOMPLETE': True,
    'RELATED_FIELD_USE_AUTOCOMPLETE': True,
    'RELATED_FIELD_FILTER_USE_AUTOCOMPLETE': True,
    'DATE_FIELD_FILTER_USE_RANGE': True,
    # APIView
    'API_EXCEPTION_DEFAULT_STATUS_CODE': 400,

    'API_SYSTEM_ERROR_STATUS_CODE': 500,
    'API_PARAM_ERROR_STATUS_CODE': 400,
    'API_SUCCESS_WITH_CODE': True,
    'API_SHOW_PARAM_ERROR_INFO': True,

    'API_ERROR_CODES': (),

    'API_CODE_MSG_IN_SUCCESS_RESPONSE': True,
    'API_DEFAULT_CODE_KEY': 'code',
    'API_DEFAULT_MESSAGE_KEY': 'message',
    'API_DEFAULT_DATA_KEY': 'data',

    'API_RESPONSE_DICT_FUNCTION': 'cool.views.response.get_response_dict',

}


# List of settings that may be in string import notation.
IMPORT_STRINGS = [
    'API_RESPONSE_DICT_FUNCTION',
]


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        return import_string(val)
    except ImportError as e:
        msg = "Could not import '%s' for API setting '%s'. %s: %s." % (val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


class CoolSettings:
    """
    A settings object that allows Django-Cool settings to be accessed as
    properties. For example:
        from cool.settings import cool_settings
        print(cool_settings.FILTER_USE_SELECT)
    Any setting with string import paths will be automatically resolved
    and return the class, rather than the string literal.
    Note:
    This is an internal class that is only compatible with settings namespaced
    under the REST_FRAMEWORK name. It is not intended to be used by 3rd-party
    apps, and test helpers like `override_settings` may not work as expected.
    """
    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        if user_settings:
            self._user_settings = user_settings
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS
        self._cached_attrs = set()

    @property
    def user_settings(self):
        if not hasattr(self, '_user_settings'):
            self._user_settings = getattr(settings, 'DJANGO_COOL', {})
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid API setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        self._cached_attrs.add(attr)
        setattr(self, attr, val)
        return val

    def reload(self):
        for attr in self._cached_attrs:
            delattr(self, attr)
        self._cached_attrs.clear()
        if hasattr(self, '_user_settings'):
            delattr(self, '_user_settings')


cool_settings = CoolSettings(None, DEFAULTS, IMPORT_STRINGS)


def reload_cool_settings(*args, **kwargs):
    setting = kwargs['setting']
    if setting == 'DJANGO_COOL':
        cool_settings.reload()


setting_changed.connect(reload_cool_settings)
